@@ -51,7 +51,6 @@ struct GsmPresencePrivate
         char             *status_text;
         gboolean          idle_enabled;
         GnomeIdleMonitor *idle_monitor;
-        guint             idle_became_active_id;
         guint             idle_watch_id;
         guint             idle_timeout;
         gboolean          screensaver_active;
@@ -134,6 +133,7 @@ set_session_idle (GsmPresence   *presence,
 
                 /* restore saved status */
                 gsm_presence_set_status (presence, presence->priv->saved_status, NULL);
+                g_debug ("GsmPresence: setting non-idle status %d", presence->priv->saved_status);
                 presence->priv->saved_status = GSM_PRESENCE_STATUS_AVAILABLE;
         }
 }
@@ -144,48 +144,50 @@ idle_became_idle_cb (GnomeIdleMonitor *idle_monitor,
                      gpointer          user_data)
 {
         GsmPresence *presence = user_data;
-        set_session_idle (presence, TRUE);
+        /* We should already be idle,
+         * see on_screensaver_active_changed() */
+        if (!presence->priv->screensaver_active)
+                set_session_idle (presence, TRUE);
 }
 
 static void
 idle_became_active_cb (GnomeIdleMonitor *idle_monitor,
                        GsmPresence      *presence)
 {
-        set_session_idle (presence, FALSE);
+        /* We can only be non-idle if the screensaver is gone */
+        if (!presence->priv->screensaver_active)
+                set_session_idle (presence, FALSE);
 }
 
 static void
 reset_idle_watch (GsmPresence  *presence)
 {
-        if (presence->priv->idle_monitor == NULL) {
-                return;
-        }
-
         if (presence->priv->idle_watch_id > 0) {
-                g_debug ("GsmPresence: removing idle watch");
+                g_debug ("GsmPresence: removing idle watch (%i)", presence->priv->idle_watch_id);
                 gnome_idle_monitor_remove_watch (presence->priv->idle_monitor,
                                                  presence->priv->idle_watch_id);
                 presence->priv->idle_watch_id = 0;
         }
 
-        if (presence->priv->idle_became_active_id > 0) {
-                g_signal_handler_disconnect (presence->priv->idle_monitor,
-                                             presence->priv->idle_became_active_id);
-                presence->priv->idle_became_active_id = 0;
-        }
-
-        if (! presence->priv->screensaver_active
-            && presence->priv->idle_enabled
+        if (presence->priv->idle_enabled
             && presence->priv->idle_timeout > 0) {
-                g_debug ("GsmPresence: adding idle watch");
-
                 presence->priv->idle_watch_id = gnome_idle_monitor_add_watch (presence->priv->idle_monitor,
                                                                               presence->priv->idle_timeout,
                                                                               idle_became_idle_cb,
                                                                               presence,
                                                                               NULL);
-                presence->priv->idle_became_active_id = g_signal_connect (presence->priv->idle_monitor, "became-active",
-                                                                          G_CALLBACK (idle_became_active_cb), presence);
+                g_debug ("GsmPresence: adding idle watch (%i) for %d secs",
+                         presence->priv->idle_watch_id,
+                         presence->priv->idle_timeout / 1000);
+
+                /* If the idle-delay is really short, we might end up with
+                 * the idletime already being past the timeout */
+                if (presence->priv->idle_timeout < gnome_idle_monitor_get_idletime (presence->priv->idle_monitor)) {
+                        g_debug ("Idle time %i already expired, triggering idle", presence->priv->idle_timeout);
+                        idle_became_idle_cb (presence->priv->idle_monitor,
+                                             presence->priv->idle_watch_id,
+                                             presence);
+                }
         }
 }
 
@@ -197,7 +199,6 @@ on_screensaver_active_changed (DBusGProxy  *proxy,
         g_debug ("screensaver status changed: %d", is_active);
         if (presence->priv->screensaver_active != is_active) {
                 presence->priv->screensaver_active = is_active;
-                reset_idle_watch (presence);
                 set_session_idle (presence, is_active);
         }
 }
@@ -211,7 +212,6 @@ on_screensaver_proxy_destroy (GObject     *proxy,
         presence->priv->screensaver_proxy = NULL;
         presence->priv->screensaver_active = FALSE;
         set_session_idle (presence, FALSE);
-        reset_idle_watch (presence);
 }
 
 static void
@@ -236,6 +236,9 @@ on_bus_name_owner_changed (DBusGProxy  *bus_proxy,
         } else if (strlen (old_service_name) == 0
                    && strlen (new_service_name) > 0) {
                 /* service added */
+
+                g_debug ("Detected that screensaver has appeared on the bus");
+
                 error = NULL;
                 presence->priv->screensaver_proxy = dbus_g_proxy_new_for_name_owner (presence->priv->bus_connection,
                                                                                      GS_NAME,
@@ -327,6 +330,8 @@ gsm_presence_init (GsmPresence *presence)
         presence->priv = GSM_PRESENCE_GET_PRIVATE (presence);
 
         presence->priv->idle_monitor = gnome_idle_monitor_new ();
+        g_signal_connect (presence->priv->idle_monitor, "became-active",
+                          G_CALLBACK (idle_became_active_cb), presence);
 }
 
 void
@@ -468,15 +473,8 @@ gsm_presence_finalize (GObject *object)
                 presence->priv->idle_watch_id = 0;
         }
 
-        if (presence->priv->status_text != NULL) {
-                g_free (presence->priv->status_text);
-                presence->priv->status_text = NULL;
-        }
-
-        if (presence->priv->idle_monitor != NULL) {
-                g_object_unref (presence->priv->idle_monitor);
-                presence->priv->idle_monitor = NULL;
-        }
+        g_clear_pointer (&presence->priv->status_text, g_free);
+        g_clear_object (&presence->priv->idle_monitor);
 
         G_OBJECT_CLASS (gsm_presence_parent_class)->finalize (object);
 }
@@ -542,7 +540,7 @@ gsm_presence_class_init (GsmPresenceClass *klass)
                                                             "idle timeout",
                                                             0,
                                                             G_MAXINT,
-                                                            300000,
+                                                            120000,
                                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
         dbus_g_object_type_install_info (GSM_TYPE_PRESENCE, &dbus_glib_gsm_presence_object_info);
