@@ -39,8 +39,6 @@
 #include <dbus/dbus-glib-bindings.h>
 #include <dbus/dbus-glib-lowlevel.h>
 
-#include <libnotify/notify.h>
-
 #include "gdm-log.h"
 
 #include "gsm-util.h"
@@ -182,6 +180,13 @@ term_or_int_signal_cb (gpointer data)
         return FALSE;
 }
 
+static gboolean
+sigusr2_cb (gpointer data)
+{
+        g_debug ("-------- MARK --------");
+        return TRUE;
+}
+
 static gboolean
 sigusr1_cb (gpointer data)
 {
@@ -297,8 +302,6 @@ main (int argc, char **argv)
                 exit (1);
         }
 
-        notify_init ("GNOME session manager");
-
         gdm_log_init ();
         gdm_log_set_debug (debug);
 
@@ -340,6 +343,7 @@ main (int argc, char **argv)
         g_unix_signal_add (SIGTERM, term_or_int_signal_cb, manager);
         g_unix_signal_add (SIGINT, term_or_int_signal_cb, manager);
         g_unix_signal_add (SIGUSR1, sigusr1_cb, manager);
+        g_unix_signal_add (SIGUSR2, sigusr2_cb, manager);
 
         if (IS_STRING_EMPTY (opt_session_name))
                 session_name = _gsm_manager_get_default_session (manager);
@@ -349,7 +353,7 @@ main (int argc, char **argv)
         gsm_util_set_autostart_dirs (override_autostart_dirs);
 
         if (!gsm_session_fill (manager, session_name)) {
-                gsm_util_init_error (TRUE, "Failed to load session \"%s\"", session_name ? session_name : "(null)");
+                gsm_fail_whale_dialog_we_failed (FALSE, TRUE, NULL);
         }
 
         gsm_manager_start (manager);
